import 'package:flutter/cupertino.dart';
import 'package:hive/hive.dart';
import 'package:kinfolk/model/url_types.dart';

import '../global_variables.dart';

class Utils {
  /// creating REST url : http://localhost:8080/test/v2/services/serviceName/methodName
  /// creating REST url : http://localhost:8080/test/v2/entities/entityName/id
  /// creating REST url : http://localhost:8080/test/v2/entities/entityName/search
  /// creating REST url : http://localhost:8080/test/v2/queries/entityName/queryName
  static String createRestUrl(
      String serviceOrEntityName, String methodNameOrEntityId, Types type) {
    String urlSuffix = UrlTypes.path[type];
    return "${GlobalVariables.urlEndPoint}/rest/v2/$urlSuffix/$serviceOrEntityName/$methodNameOrEntityId";
  }
}

class HiveService {
  static void regAdapters(List<TypeAdapter> list) {
    list.forEach((element) => Hive.registerAdapter(element));
  }

  static getBox(String name) async {
    Box box;
    if (!Hive.isBoxOpen(name)) {
      box = await Hive.openBox(name);
    } else {
      box = Hive.box(name);
    }
    return box;
  }

  static getClearBox(String name) async {
    Box box = await getBox(name);
    await box.clear().catchError((val) => print(val));
    return box;
  }
}

class DBProvider {
  static Future<bool> saveEntity({element, String boxName, String id}) async {
    Box box = await HiveService.getBox(boxName);
    await box.add(element);
    return true;
  }

  static Future<bool> saveEntityById(
      {element, String boxName, String id}) async {
    Box box = await HiveService.getBox(boxName);
    await box.put(id ?? element.id, element);
    return true;
  }

  static Future<bool> saveEntitiesList({List list, String boxName}) async {
    Box box = await HiveService.getBox(boxName);
    box.addAll(list);
    return true;
  }

  static Future<bool> saveEntitiesListById({List list, String boxName}) async {
    Box box = await HiveService.getBox(boxName);
    list.forEach((element) async => await box.put(element.id, element));
    return true;
  }

  static Future<List> getEntities({@required String boxName}) async {
    Box box = await HiveService.getBox(boxName);
    var list = box.values.toList();
    return list;
  }
}
